Public Class NetlabNetwork
    Public Enum NetLoadStatus
        status_Ok
        status_Canceled
        status_Error
    End Enum

    ' Node and link data.
    Private m_Nodes As New Collection()
    Private m_Links As New Collection()

    ' Creates a new node, adds it to the m_Nodes collection, 
    ' and returns a reference to the new node.
    Public Function AddNode(Optional ByVal X As Integer = 0, Optional ByVal Y As Integer = 0) As Node
        Dim new_node As New Node(X, Y)

        m_Nodes.Add(new_node)
        Return new_node
    End Function

    ' Creates a new link, adds it to the m_Links collection, 
    ' and returns a reference to the new link.
    Public Function AddLink(ByVal node1 As Node, ByVal node2 As Node) As Link
        Dim new_link As New Link(node1, node2)

        m_Links.Add(new_link)
        Return new_link
    End Function

    ' Removes the node from the m_Nodes collection. It also removes
    ' the links that start and end at the node from the m_Links 
    ' collection.
    Public Sub DeleteNode(ByVal target_node As Node)
        Dim i As Integer

        ' Examine each link. We examine them in reverse order so
        ' removing a link does not change the numbering of those
        ' that remain.
        For i = m_Links.Count To 1 Step -1
            With CType(m_Links(i), Link)
                ' If this link starts or ends at the target node ...
                If (.Node1 Is target_node) Or (.Node2 Is target_node) Then
                    ' ... remove the link.
                    m_Links.Remove(i)
                End If
            End With
        Next i

        ' Remove the node.
        ' Find the node.
        For i = 1 To m_Nodes.Count
            ' If this is the node ...
            If m_Nodes(i) Is target_node Then
                ' ... remove the node.
                m_Nodes.Remove(i)
                Exit For
            End If
        Next i
    End Sub

    ' Removes the link from the m_Links collection.
    Public Sub DeleteLink(ByVal target_link As Link)
        Dim i As Integer

        ' Find the link.
        For i = 1 To m_Links.Count
            ' If this is the link ...
            If m_Links(i) Is target_link Then
                ' ... remove the link.
                m_Links.Remove(i)
                Exit For
            End If
        Next i
    End Sub

    ' Return the node nearest to the indicated point.
    Public Function FindNearestNode(ByVal X As Integer, ByVal Y As Integer) As Node
        Dim nearest_node As Node = Nothing
        Dim nearest_distance As Double = 1.79769313486231E+308
        Dim test_node As Node
        Dim test_distance As Double

        For Each test_node In m_Nodes
            test_distance = test_node.DistanceToPoint(X, Y)
            If nearest_distance > test_distance Then
                nearest_distance = test_distance
                nearest_node = test_node
            End If
        Next test_node

        ' Return the best node we found.
        Return nearest_node
    End Function

    ' Clears the drawing surface and then loops through the 
    ' m_Links collection drawing the links. It then loops through 
    ' m_Nodes draws the nodes.
    Public Sub Draw(ByVal canvas As System.Drawing.Graphics)
        Dim test_link As Link
        Dim test_node As Node

        ' Clear the drawing area.
        canvas.Clear(SystemColors.Control)

        ' Make the links draw themselves.
        For Each test_link In m_Links
            test_link.Draw(canvas)
        Next test_link

        ' Make the nodes draw themselves.
        For Each test_node In m_Nodes
            test_node.Draw(canvas)
        Next test_node
    End Sub

    ' Return the object the mouse is over.
    Public Function MouseIsOver(ByVal X As Integer, ByVal Y As Integer) As Object
        Dim i As Integer
        Dim test_node As Node
        Dim test_link As Link

        ' See if the mouse is over a node.
        ' Check nodes first because the sit on top of links.
        ' Check them in reverse order so the ones drawn on top
        ' get checked first.
        For i = m_Nodes.Count To 1 Step -1
            test_node = CType(m_Nodes.Item(i), Node)
            If test_node.IsAt(X, Y) Then Return test_node
        Next i

        ' See if the mouse is over a link.
        ' Check them in reverse order so the ones drawn on top
        ' get checked first.
        For i = m_Links.Count To 1 Step -1
            test_link = CType(m_Links.Item(i), Link)
            If test_link.IsAt(X, Y) Then Return test_link
        Next i

        Return Nothing
    End Function

    ' *******************************
    ' Routines to make test networks.
    ' *******************************
    Public Sub MakeGridNetwork( _
      ByVal wid As Integer, ByVal hgt As Integer, _
      ByVal num_rows As Integer, ByVal num_columns As Integer, _
      ByVal express_links As Integer)
        Dim r As Integer
        Dim c As Integer
        Dim dist As Integer
        Dim node1 As Node
        Dim node2 As Node
        Dim rnd As New System.Random()
        Dim num_nodes As Integer

        ' Calculate a good distance between nodes.
        If wid / num_columns < hgt / num_rows Then
            dist = wid \ (num_columns + 1)
        Else
            dist = hgt \ (num_rows + 1)
        End If

        ' Make the nodes.
        num_nodes = num_rows * num_columns
        m_Nodes = New Collection()
        For r = 0 To num_rows - 1
            For c = 0 To num_columns - 1
                m_Nodes.Add(New Node((c + 1) * dist, (r + 1) * dist))
            Next c
        Next r

        ' Make the links.
        m_Links = New Collection()
        ' Horizontal links.
        For r = 0 To num_rows - 1
            For c = 0 To num_columns - 2
                node1 = CType(m_Nodes(1 + r * num_columns + c), Node)
                node2 = CType(m_Nodes(1 + r * num_columns + c + 1), Node)
                m_Links.Add(New Link(node1, node2))
            Next c
        Next r
        ' Vertical links.
        For c = 0 To num_columns - 1
            For r = 0 To num_rows - 2
                node1 = CType(m_Nodes(1 + r * num_columns + c), Node)
                node2 = CType(m_Nodes(1 + r * num_columns + c + num_columns), Node)
                m_Links.Add(New Link(node1, node2))
            Next r
        Next c
        ' Express links.
        For r = 1 To express_links
            node1 = CType(m_Nodes(CInt(1 + Int(rnd.NextDouble * num_nodes))), Node)
            node2 = CType(m_Nodes(CInt(1 + Int(rnd.NextDouble * num_nodes))), Node)
            m_Links.Add(New Link(node1, node2))
        Next r
    End Sub

    ' Save the network data into the indicated file.
    Public Sub Save(ByVal file_name As String)
        Dim xml_document As New Xml.XmlDocument()
        Dim xml_declaration As Xml.XmlDeclaration
        Dim xml_comment As Xml.XmlComment
        Dim root_node As Xml.XmlNode
        Dim node_attribute As Xml.XmlAttribute
        Dim parent_node As Xml.XmlNode
        Dim child_node As Xml.XmlNode
        Dim i As Integer
        Dim test_node As Node
        Dim test_link As Link

        ' Convert the network data into an XML document.
        ' Create an empty document.
        xml_declaration = xml_document.CreateXmlDeclaration("1.0", "", "yes")
        xml_document.AppendChild(xml_declaration)

        xml_comment = xml_document.CreateComment("Network data created by VB NetLab 0.03")
        xml_document.AppendChild(xml_comment)

        xml_comment = xml_document.CreateComment(Now().ToString())
        xml_document.AppendChild(xml_comment)

        ' Create the root node.
        root_node = xml_document.CreateElement("VBNetLab_Network")
        node_attribute = xml_document.CreateAttribute("Version")
        node_attribute.Value = "0.03"
        root_node.Attributes.Append(node_attribute)
        xml_document.AppendChild(root_node)

        ' Make entries for the nodes.
        parent_node = xml_document.CreateElement("Nodes")
        root_node.AppendChild(parent_node)
        i = 1
        For Each test_node In m_Nodes
            ' Make the Node's XML node.
            child_node = xml_document.CreateElement("Node")
            parent_node.AppendChild(child_node)

            ' Save the node's X and Y values as attributes.
            node_attribute = xml_document.CreateAttribute("X")
            node_attribute.Value = test_node.X.ToString()
            child_node.Attributes.Append(node_attribute)

            node_attribute = xml_document.CreateAttribute("Y")
            node_attribute.Value = test_node.Y.ToString()
            child_node.Attributes.Append(node_attribute)

            ' Set the node's index.
            test_node.XmlId = i
            i = i + 1
        Next test_node

        ' Make entries for the links.
        parent_node = xml_document.CreateElement("Links")
        root_node.AppendChild(parent_node)
        For Each test_link In m_Links
            ' Make the Link's XML node.
            child_node = xml_document.CreateElement("Link")
            parent_node.AppendChild(child_node)

            ' Save the indexes of the Nodes that 
            ' this Link connects.
            node_attribute = xml_document.CreateAttribute("Node1")
            node_attribute.Value = test_link.Node1.XmlId.ToString()
            child_node.Attributes.Append(node_attribute)

            node_attribute = xml_document.CreateAttribute("Node2")
            node_attribute.Value = test_link.Node2.XmlId.ToString()
            child_node.Attributes.Append(node_attribute)

            node_attribute = xml_document.CreateAttribute("Cost")
            node_attribute.Value = test_link.Cost.ToString()
            child_node.Attributes.Append(node_attribute)
        Next test_link

        xml_document.Save(file_name)
    End Sub

    ' Load the network data from the indicated file. 
    ' Return True if there is a problem, False otherwise.
    Public Function Load(ByVal file_name As String) As Boolean
        Dim xml_document As New Xml.XmlDocument()
        Dim xml_attributes As Xml.XmlAttributeCollection
        Dim root_node As Xml.XmlNode
        Dim node_attribute As Xml.XmlAttribute
        Dim parent_node As Xml.XmlNode
        Dim child_node As Xml.XmlNode
        Dim children As Xml.XmlNodeList
        Dim i As Integer
        Dim new_node As Node
        Dim new_link As Link
        Dim new_node1 As Node
        Dim new_node2 As Node
        Dim new_cost As Double

        ' Load the document.
        xml_document.Load(file_name)

        ' Verify that the file was written by VB NetLab 0.03.
        root_node = xml_document.DocumentElement()
        xml_attributes = root_node.Attributes()
        Debug.WriteLine(xml_attributes.GetNamedItem("Version").Value) '@
        If xml_attributes.GetNamedItem("Version").Value <> "0.03" Then
            If MsgBox("This file was written with VB NetLab " & _
                xml_attributes("Version").Value & _
                ". Some data may be read incorrectly." & vbCrLf & vbCrLf & "Continue anyway?", _
                Microsoft.VisualBasic.MsgBoxStyle.YesNo, _
                "Warning") = MsgBoxResult.No _
            Then
                ' Return a failure code.
                Return False
            End If
        End If

        ' Empty the m_Nodes and m_Links collections.
        m_Nodes = New Collection()
        m_Links = New Collection()

        ' Load the nodes.
        parent_node = root_node.ChildNodes(0)
        children = parent_node.ChildNodes()
        i = 1
        For Each child_node In children
            new_node = New Node( _
                CInt(child_node.Attributes.GetNamedItem("X").Value), _
                CInt(child_node.Attributes.GetNamedItem("Y").Value))
            m_Nodes.Add(new_node)
        Next child_node

        ' Load the links.
        parent_node = root_node.ChildNodes(1)
        children = parent_node.ChildNodes()
        For Each child_node In children
            new_node1 = CType(m_Nodes(CInt(child_node.Attributes.GetNamedItem("Node1").Value)), Node)
            new_node2 = CType(m_Nodes(CInt(child_node.Attributes.GetNamedItem("Node2").Value)), Node)
            new_cost = CDbl(child_node.Attributes.GetNamedItem("Cost").Value)
            new_link = New Link(new_node1, new_node2, new_cost)
            m_Links.Add(new_link)
        Next child_node

        ' Return a success code.
        Return False
    End Function

    ' Return the number of Nodes in the network.
    Public Function NumNodes() As Integer
        Return m_Nodes.Count
    End Function

    ' Return the number of Links in the network.
    Public Function NumLinks() As Integer
        Return m_Links.Count
    End Function
End Class
